//
//  DialecticController.h
//  Dialectic
//
//  Created by Jon Nathan, JNSoftware LLC.
//  Copyright 2015 JNSoftware LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@protocol DialecticControllerDelegate;

@interface DialecticController : NSObject {
	//set a delegate to be notified of the outgoing & incoming call events if Dialectic is configured to broadcast these events
	id <DialecticControllerDelegate> delegate;
}

@property (nonatomic, assign) id <DialecticControllerDelegate> delegate;

/*
 sharedDialecticController
 standard Dialectic controller
 */
+ (id)sharedDialecticController;

/*
 dialecticBundleIdentifier
 will return the bundle identifier of the Dialectic application regardless of whether Dialectic is installed or not
 */
- (NSString *)dialecticBundleIdentifier;

/*
 dialecticSignature
 will return the application signature of the Dialectic application regardless of whether Dialectic is installed or not
 */
- (NSString *)dialecticSignature;

/*
 dialecticPath
 if Dialectic is not installed, will return nil otherwise it will return the path to the Dialectic application
 */
- (NSString *)dialecticPath;

/*
 dialecticVersion
 if Dialectic is not installed, will return nil otherwise it will return the version of the Dialectic application
 */
- (NSString *)dialecticVersion;

/*
 dialecticLatestVersion
 will attempt to retreive the latest version string of Dialectic from Dialectic's online repository; will return nil if unsuccessful
 */
- (NSString *)dialecticLatestVersion;

/*
 dialecticUpdateAvailable
 will attempt to get the latest version string from online and compare that to the version string of the installed version of Dialectic
 if the latest version string is newer than the installed, will return true
 if Dialectic is not installed on this machine or if the retreival of the latest version string fails will return false
 if the installed version is the same or newer than the latest version string retreived from the web, will return false
 */
- (BOOL)dialecticUpdateAvailable;

/*
 dialecticCopyright
 if Dialectic is not installed, will return nil otherwise it will return the copyright string of the Dialectic application
 */
- (NSString *)dialecticCopyright;

/*
 dialecticProductWebpageURL
 will return the Dialectic product webpage URL regardless of whether Dialectic is installed or not
 */
- (NSURL *)dialecticProductWebpageURL;

/*
 dialecticIcon
 will return the icon of the Dialectic application regardless of whether Dialectic is installed or not
 */
- (NSImage *)dialecticIcon;

/*
 isDialecticInstalled
 is Dialectic installed on this system?
 */
- (BOOL)isDialecticInstalled;

/*
 isDialecticRunning
 is Dialectic currently running on this system?
 */
- (BOOL)isDialecticRunning;

/*
 isDialecticDialing
 is Dialectic currently dialing?
 */
- (BOOL)isDialecticDialing;

/*
 isDialecticVerboseLogging
 is Dialectic verbose logging?
 */
- (BOOL)isDialecticVerboseLogging;

/*
 isDialecticInBackgroundMode
 is Dialectic in background mode? (background mode means no Dock icon; menubar only)
 */
- (BOOL)isDialecticInBackgroundMode;

/*
 activateDialectic
 will return NO if Dialectic is not installed
 will launch Dialectic if it is not already running
 */
- (BOOL)activateDialectic;

/*
 revealDialectic
 will return NO if Dialectic is not installed
 will reveal Dialectic in the Finder
 */
- (BOOL)revealDialectic;

/*
 dialecticCallLogEntries
 if Dialectic is not running, will return nil otherwise it will return an array of the call log entries (dictionaries)
 */
- (NSArray *)dialecticCallLogEntries;

/*
 dialecticQuickDialNumbers
 if Dialectic is not running, will return nil otherwise it will return an array of the quick dial numbers (dictionaries)
 */
- (NSArray *)dialecticQuickDialNumbers;

/*
 dialecticRecentlyDialedNumbers
 if Dialectic is not running, will return nil otherwise it will return an array of the recently dialed numbers (dictionaries)
 */
- (NSArray *)dialecticRecentlyDialedNumbers;

/*
 dialecticLocations
 if Dialectic is not running, will return nil otherwise it will return an array of the locations (dictionaries)
 */
- (NSArray *)dialecticLocations;

/*
 dialecticLocationNames
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the location names (strings)
 */
- (NSArray *)dialecticLocationNames;

/*
 currentDialecticLocation
 if Dialectic is not running, will return nil otherwise it will return the current location (dictionary)
 */
- (NSDictionary *)currentDialecticLocation;

/*
 currentDialecticLocationName
 if Dialectic is not running, will return nil otherwise it will return the current location name (string)
 */
- (NSString *)currentDialecticLocationName;

/*
 setCurrentDialecticLocation
 allows you to set the current location in Dialectic
 if Dialectic is not running, will return false
 if the location name is not the name of an existing location, will return false
 */
- (BOOL)setCurrentDialecticLocation:(NSString *)locationName;

/*
 dialecticDialMethodConfigurations
 if Dialectic is not running, will return nil otherwise it will return an array of all dial method configurations (dictionaries)
 */
- (NSArray *)dialecticDialMethodConfigurations;

/*
 dialecticDialMethodConfigurationNames
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the dial method configuration names (strings)
 */
- (NSArray *)dialecticDialMethodConfigurationNames;

/*
 dialecticDevices
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the device (i.e., modem) names (strings)
 */
- (NSArray *)dialecticDevices;

/*
 dialecticBluetoothDevices
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the Bluetooth device (i.e., phone) names (strings)
 */
- (NSArray *)dialecticBluetoothDevices;

/*
 dialecticSoftphones
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the installed, supported softphone application names (strings)
 */
- (NSArray *)dialecticSoftphones;

/*
 dialecticLookupSources
 if Dialectic is not running, will return nil otherwise it will return an alphabetized array of the installed, supported application names available as the source for lookups (strings)
 */
- (NSArray *)dialecticLookupSources;

/*
 dialDialectic:name:literal:identifier:
 basic dial command with minimal error checking
 will return NO if Dialectic is not installed or not running
 will return NO if contactNumber is nil or empty string (@"")
 contactNumber: (i.e., phone number) is required
 contactName: (used for a label in the call log) is optional
 literal: whether or not the number should be dialed as formatted (literal = YES) or if it should be processed by Dialectic's current dialing rules (literal = NO)
 identifier: an optional string you can use to determine call status later via a callback
 */
- (BOOL)dialDialectic:(NSString *)contactNumber name:(NSString *)contactName literal:(BOOL)literal identifier:(NSString *)identifier;

/*
 dialDialectic:callData:
 will return NO if Dialectic is not installed or not running
 will return NO if callData is nil or does not contain valid phone number for DialecticContactNumberKey
 callData must contain a valid phone number for the DialecticContactNumberKey key/value pair
 callData can optionally contain a contact name string for the DialecticContactNameKey key/value pair
 callData can optionally contain a known Dialectic location name string for the DialecticLocationNameKey key/value pair; if supplied, this location will be used for the current call; if not valid, an error will be returned
 callData can optionally contain a boolean number for the DialecticLiteralKey key/value pair; if supplied and the value is YES, Dialectic will dial the phone number passed as is without applying additional formatting or the current dialing rules
 callData can optionally contain a unique string for the DialecticIdentifierKey key/value pair; this can be used for callbacks to act on a call later when notified of the dialing success or failure
 will return YES if the dial command is accepted, even if the call fails; to determine the call success, listen for the outgoing call notification (only posted if Dialectic is configured to broadcast incoming calls)
 basic example:
 
 [[DialecticController sharedDialecticController] dialDialectic:[NSDictionary dictionaryWithObjectsAndKeys:@"2125551212", DialecticContactNumberKey, nil] error:nil];
 
 complex example:
 
 NSString *contactNumber = @"(212) 555-5555"; //required; a valid phone number string
 NSString *contactName = @"Jon Smith"; //optional; anything; may be overridden by Dialectic if name lookup is enabled in Dialectic and a different name is returned from the lookup source
 NSString *locationName = @"Home"; //optional; if included, must be the exact name of an existing Dialectic location
 NSNumber *literal = [NSNumber numberWithBool:NO]; //optional; if included, must be YES or NO NSNumber
 NSString *identifier = @""; //optional; any string (if blank, a UUID will be generated for the call)
 NSDictionary *callData = [NSDictionary dictionaryWithObjectsAndKeys:
 contactNumber, DialecticContactNumberKey, //required
 contactName, DialecticContactNameKey, //optional
 locationName, DialecticLocationNameKey, //optional
 literal, DialecticLiteralKey, //optional
 identifier, DialecticIdentifierKey, //optional
 nil];
 NSError *error = nil
 BOOL success = [[DialecticController sharedDialecticController] dialDialectic:callData error:&error];
 if (error) NSLog(@"dialDialectic error: %@", [error localizedDescription]);
 NSLog(@"dialDialectic success: %d", success);
 */
- (BOOL)dialDialectic:(NSDictionary *)callData error:(NSError **)error;

/*
 notifyDialecticOfIncomingCall:name:device:note:flagged:identifier:
 basic call notification with minimal error checking
 will fail if Dialectic is not configured to allow external AppleScript access to logging and status features
 will return NO if Dialectic is not installed or not running
 contactNumber: (i.e., phone number) is optional
 contactName: (used for a label in the call log) is optional
 device: the name of phone or service that received the call (e.g., @"My Phone") is optional
 note: a note to be included in the call log is optional
 flagged: a boolean to indicate whether the call log entry for this call should be flagged (flagged = YES)
 identifier: an optional string you can use to determine call status later via a callback
 */
- (BOOL)notifyDialecticOfIncomingCall:(NSString *)contactNumber name:(NSString *)contactName device:(NSString *)device note:(NSString *)note flagged:(BOOL)flagged identifier:(NSString *)identifier;

/*
 notifyDialecticOfIncomingCall:error:
 will fail if Dialectic is not configured to allow external AppleScript access to logging and status features
 will return NO if Dialectic is not installed or not running
 callData can optionally contain a valid phone number for the DialecticContactNumberKey key/value pair
 callData can optionally contain a contact name string for the DialecticContactNameKey key/value pair
 callData can optionally contain a device string for the DialecticDeviceKey key/value pair; (e.g., the name of the phone, PBX, or service that received the call) 
 callData can optionally contain a note string for the DialecticNoteKey key/value pair; used for the call log entry for this call
 callData can optionally contain a boolean number for the DialecticFlaggedKey key/value pair; if supplied and the value is YES, Dialectic will flag the call log entry for this call
 callData can optionally contain a unique string for the DialecticIdentifierKey key/value pair; this can be used for callbacks to act on a call later
 basic examples:
 
 [[DialecticController sharedDialecticController] notifyDialecticToIncomingCall:nil error:nil];
 [[DialecticController sharedDialecticController] notifyDialecticToIncomingCall:[NSDictionary dictionaryWithObjectsAndKeys:@"2125551212", DialecticContactNumberKey, nil] error:nil];
 
 complex example:
 
 NSString *contactNumber = @"(212) 555-5555"; //optional; a valid phone number string
 NSString *contactName = @"Jon Smith"; //optional; anything; may be overridden by Dialectic if name lookup is enabled in Dialectic and a different name is returned from the lookup source
 NSString *device = @"My Phone"; //optional; anything but should be the name of the name of the phone, PBX, or service that received the call
 NSString *note = @"A note..."; //optional; anything; used for the call log entry for this call
 NSNumber *flagged = [NSNumber numberWithBool:NO]; //optional; if included, must be a YES or NO NSNumber, will be used to flag (or not) the call log entry for this call
 NSString *identifier = @""; //optional; any string (if blank, a UUID will be generated for the call)
 NSDictionary *callData = [NSDictionary dictionaryWithObjectsAndKeys:
 contactNumber, DialecticContactNumberKey, //optional
 contactName, DialecticContactNameKey, //optional
 device, DialecticDeviceKey, //optional
 note, DialecticNoteKey, //optional
 flagged, DialecticFlaggedKey, //optional
 identifier, DialecticIdentifierKey, //optional
 nil];
 NSError *error = nil
 BOOL success = [[DialecticController sharedDialecticController] notifyDialecticToIncomingCall:callData error:&error];
 if (error) NSLog(@"notifyDialecticToIncomingCall error: %@", [error localizedDescription]);
 NSLog(@"notifyDialecticToIncomingCall success: %d", success);
 */
- (BOOL)notifyDialecticOfIncomingCall:(NSDictionary *)callData error:(NSError **)error;

/*
 addDialecticCallLogEntryNumber:name:incoming:note:flagged:identifier:
 similar to notifying Dialectic to an incoming call, this command allows you to add a call log entry without executing all the other incoming call actions
 will fail if Dialectic is not configured to allow external AppleScript access to logging and status features
 will return NO if Dialectic is not installed or not running
 contactNumber: (i.e., phone number) is required
 contactName: (used for a label in the call log) is optional
 incoming: a boolean to indicate if the call was an incoming or outgoing call
 device: the name of phone or service that received the call (e.g., @"My Phone") is optional
 note: a note to be included in the call log is optional
 flagged: a boolean to indicate whether the call log entry for this call should be flagged (flagged = YES)
 identifier: an optional string you can use to determine call status later via a callback
 */
- (BOOL)addDialecticCallLogEntryNumber:(NSString *)contactNumber name:(NSString *)contactName incoming:(BOOL)incoming note:(NSString *)note flagged:(BOOL)flagged identifier:(NSString *)identifier;

/*
 lookupContactInDialectic:lookupSource:companiesOnly:
 will do nothing if Dialectic is not installed or not running
 contactName: the name to lookup is required
 lookupSource: the source of the lookup (use dialecticLookupSources to see valid sources) is optional; pass nil to use the current lookup source
 */
- (void)lookupContactInDialectic:(NSString *)contactName lookupSource:(NSString *)lookupSource;

/*
 clearDialecticRecentlyDialedNumbers
 will return NO if Dialectic is not installed or not running
 this command will clear the recently dialed numbers history though the calls will still be listed in the Dialectic call log
 */
- (BOOL)clearDialecticRecentlyDialedNumbers;

/*
 convertMnemonic:
 will return nil if Dialectic is not installed or not running
 mnemonic: a string to convert to a phone number (e.g., 1-800-NUMBER -> 1-800-686237)
 */
- (NSString *)convertMnemonic:(NSString *)mnemonic;

/*
 showLargeType:
 requires that Dialectic allow external AppleScript access in the Miscellaneous preferences section
 if Dialectic is not running, will return NO otherwise it will show a message in large type and return YES
 */
- (BOOL)showLargeType:(NSString *)message;

/*
 showMessage:icon:
 requires that Dialectic allow external AppleScript access in the Miscellaneous preferences section
 if Dialectic is not running, will return NO otherwise it will show a message in a HUD and return YES
 message: a required string
 icon: an optional path to the image file you want to use for the icon (an a non-image path is supplied, the icon for that item will be used)
 */
- (BOOL)showMessage:(NSString *)message icon:(NSString *)icon;

/*
 playTone:duration:
 will return NO if Dialectic is not installed or not running
 tone: the touch tone to play (e.g., 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, *, or #)
 duration: the length of time (in seconds) to play the tone
 */
- (BOOL)playTone:(NSString *)tone duration:(double)duration;

/*
 startDialecticTimerWithProperties:
 will return NO if Dialectic is not installed or not running
 will start the Dialectic call timer
 properties is a dictionary that can contain any (or none) of the following key/value pairs:
 frequency (float) : How often (in minutes) should the alert display? e.g., 1 or 2.5.
 beep (boolean) : Should the system beep when the timer is up?
 reset (boolean) : Should the timer be reset when started?
 e.g.:
 NSDictionary *properties = [NSDictionary dictionaryWithObjectsAndKeys:
 [NSNumber numberWithFloat:3.0], @"frequency", 
 [NSNumber numberWithBool:NO], @"beep", 
 [NSNumber numberWithBool:YES], @"reset", 
 nil];
 [[DialecticController sharedDialecticController] startDialecticTimerWithProperties:properties];
 */
- (BOOL)startDialecticTimerWithProperties:(NSDictionary *)properties;

/*
 stopDialecticTimer
 will return NO if Dialectic is not installed or not running
 stops the Dialectic call timer
 */
- (BOOL)stopDialecticTimer;

/*
 stopDialecticTimer
 will return NO if Dialectic is not installed or not running
 opens the Dialectic AB window and activates Dialectic to bring the window to the front
 */
- (BOOL)showDialecticABWindow;

/*
 showDialecticCallLog
 will return NO if Dialectic is not installed or not running
 opens the Dialectic call log window and activates Dialectic to bring the window to the front
 */
- (BOOL)showDialecticCallLog;

/*
 showDialecticCallLogEntry
 will return NO if Dialectic is not installed or not running
 opens the Dialectic call log window and shows the specific call log entry specified by index and activates Dialectic to bring the window to the front
 */
- (BOOL)showDialecticCallLogEntry:(NSInteger)index;

/*
 showDialecticNotepad
 will return NO if Dialectic is not installed or not running
 opens the Dialectic notepad window and activates Dialectic to bring the window to the front
 */
- (BOOL)showDialecticNotepad;

/*
 showDialecticPreferences
 will return NO if Dialectic is not installed or not running
 opens the Dialectic preferences window and activates Dialectic to bring the window to the front
 */
- (BOOL)showDialecticPreferences;

/*
 showDialecticDialMethodConfiguration:
 will return NO if Dialectic is not installed or not running
 opens the Dialectic preferences window, changes the tab to the Dial Methods pane and activates Dialectic to bring the window to the front
 dialMethodConfigurationName should be the name of an existing Dialectic dial method configuration (use dialecticDialMethodConfigurationNames to get valid names)
 */
- (BOOL)showDialecticDialMethodConfiguration:(NSString *)dialMethodConfigurationName;

/*
 createNewDialecticWYWONote
 will return NO if Dialectic is not installed or not running
 opens a new While You Were Out note window and activates Dialectic to bring the window to the front
 */
- (BOOL)createNewDialecticWYWONote;

/*
 initializeModem:using:
 will return NO if Dialectic is not installed or not running
 modemName: the name of the modem to initialize (e.g., usbmodem), must be a known modem (use dialecticDevices for valid device names)
 initString: the init string (e.g., "AT&F") to use to initialize the modem, OK to pass nil
 */
- (BOOL)initializeModem:(NSString *)modemName using:(NSString *)initString;

/*
 takeOffHookModem:
 will return NO if Dialectic is not installed or not running
 modemName: the name of the modem to take off-hook (e.g., usbmodem), must be a known modem (use dialecticDevices for valid device names)
 */
- (BOOL)takeOffHookModem:(NSString *)modemName;

/*
 putOnHookModem:
 will return NO if Dialectic is not installed or not running
 modemName: the name of the modem to put on-hook (e.g., usbmodem), must be a known modem (use dialecticDevices for valid device names)
 */
- (BOOL)putOnHookModem:(NSString *)modemName;

/*
 hangUpModem:
 will return NO if Dialectic is not installed or not running
 modemName: the name of the modem to hang-up (e.g., usbmodem), must be a known modem (use dialecticDevices for valid device names)
 */
- (BOOL)hangUpModem:(NSString *)modemName;

/*
 releaseModem:
 will return NO if Dialectic is not installed or not running
 modemName: the name of the modem to release (e.g., usbmodem), must be a known modem (use dialecticDevices for valid device names)
 */
- (BOOL)releaseModem:(NSString *)modemName;

/*
 pauseDialecticIncomingCallDetection
 will return NO if Dialectic is not installed or not running
 pauses incoming call detection for all monitored modems and Bluetooth phones; can be helpful if you need access to a monitored device for some other purpose (e.g. release the modem so you can send a fax)
 */
- (BOOL)pauseDialecticIncomingCallDetection;

/*
 restoreDialecticIncomingCallDetection
 will return NO if Dialectic is not installed or not running
 resumes incoming call detection for all monitored modems and Bluetooth phones
 */
- (BOOL)restoreDialecticIncomingCallDetection;

/*
 restoreIAppStatus
 if Dialectic modified iTunes, DVD Player, or iChat when dialing or receiving an incoming call, this command will revert these apps to their pre-call state
 */
- (BOOL)restoreIAppStatus;

@end

@protocol DialecticControllerDelegate <NSObject>
@optional
//incoming call notifications only enabled if Dialectic is configured to broadcast incoming calls
- (void)dialecticIncomingCallDidStart:(NSNotification *)notification;
- (void)dialecticIncomingCallDidEnd:(NSNotification *)notification;
//outgoing call notifications only enabled if Dialectic is configured to broadcast outgoing calls
- (void)dialecticOutgoingCallDidStart:(NSNotification *)notification;
- (void)dialecticOutgoingCallDidEnd:(NSNotification *)notification;
@end

//keys
extern NSString *const DialecticActiveKey;
extern NSString *const DialecticAddLocalAreaCodeKey;
extern NSString *const DialecticAddressActionKey;
extern NSString *const DialecticAdjustITunesVolumeKey;
extern NSString *const DialecticAdjustScreenSaverKey;
extern NSString *const DialecticAdjustSystemVolumeKey;
extern NSString *const DialecticAlternateMobileDialStringKey;
extern NSString *const DialecticAutostartKey;
extern NSString *const DialecticBeepKey;
extern NSString *const DialecticBroadcastIncomingCallsKey;
extern NSString *const DialecticBroadcastOutgoingCallsKey;
extern NSString *const DialecticCallerIDKey;
extern NSString *const DialecticCallerIDBlockPrefixKey;
extern NSString *const DialecticCallerIDSearchSourceKey;
extern NSString *const DialecticCallingTonesDisabledKey;
extern NSString *const DialecticCallWaitingPrefixKey;
extern NSString *const DialecticChangeIChatStatusKey;
extern NSString *const DialecticCommaPauseKey;
extern NSString *const DialecticConnectedDeviceTypeKey;
extern NSString *const DialecticConnectionTimeoutKey;
extern NSString *const DialecticContactNameKey;
extern NSString *const DialecticContactNumberKey;
extern NSString *const DialecticContextKey;
extern NSString *const DialecticCountryKey;
extern NSString *const DialecticCreationDateKey;
extern NSString *const DialecticCurrentKey;
extern NSString *const DialecticDataItemOrderKey;
extern NSString *const DialecticDateKey;
extern NSString *const DialecticDefaultLocationKey;
extern NSString *const DialecticDefaultDialPortKey;
extern NSString *const DialecticDelayDialKey;
extern NSString *const DialecticDeviceKey;
extern NSString *const DialecticDialExactKey;
extern NSString *const DialecticDialMethodNameKey;
extern NSString *const DialecticDialSpeedKey;
extern NSString *const DialecticDirectionKey;
extern NSString *const DialecticDisplayLastNameFirstKey;
extern NSString *const DialecticDomesticDialMethodKey;
extern NSString *const DialecticDomesticPrefixKey;
extern NSString *const DialecticDomesticSuffixKey;
extern NSString *const DialecticEmailActionKey;
extern NSString *const DialecticEncodingKey;
extern NSString *const DialecticEndDateKey;
extern NSString *const DialecticExtensionKey;
extern NSString *const DialecticExternalApplicationPathKey;
extern NSString *const DialecticFlaggedKey;
extern NSString *const DialecticFrequencyKey;
extern NSString *const DialecticGroupActionKey;
extern NSString *const DialecticGroupEmailFieldKey;
extern NSString *const DialecticHangUpTimeoutKey;
extern NSString *const DialecticIChatStatusKey;
extern NSString *const DialecticIdKey;
extern NSString *const DialecticIdentifierKey;
extern NSString *const DialecticIncludeAddressDataItemKey;
extern NSString *const DialecticIncludeAllSubmenuKey;
extern NSString *const DialecticIncludeAlphabeticalSubmenuKey;
extern NSString *const DialecticIncludeCompaniesSubmenuKey;
extern NSString *const DialecticIncludeCompanyNamesKey;
extern NSString *const DialecticIncludeEmailDataItemKey;
extern NSString *const DialecticIncludeGeographicalSubmenuKey;
extern NSString *const DialecticIncludeGroupsSubmenuKey;
extern NSString *const DialecticIncludeIconsKey;
extern NSString *const DialecticIncludeInstantMessagingDataItemKey;
extern NSString *const DialecticIncludeInternationalDelimiterKey;
extern NSString *const DialecticIncludeLocalAreaCodeKey;
extern NSString *const DialecticIncludeNoteDataItemKey;
extern NSString *const DialecticIncludePhoneDataItemKey;
extern NSString *const DialecticIncludeRecentlyUsedSubmenuKey;
extern NSString *const DialecticIncludeTitlesKey;
extern NSString *const DialecticIncludeUrlDataItemKey;
extern NSString *const DialecticIndexKey;
extern NSString *const DialecticInitStringKey;
extern NSString *const DialecticInstantMessagingActionKey;
extern NSString *const DialecticInternalDialMethodKey;
extern NSString *const DialecticInternalPrefixKey;
extern NSString *const DialecticInternalSuffixKey;
extern NSString *const DialecticInternationalDelimiterKey;
extern NSString *const DialecticInternationalDialMethodKey;
extern NSString *const DialecticInternationalPrefixKey;
extern NSString *const DialecticInternationalSuffixKey;
extern NSString *const DialecticIPAddressKey;
extern NSString *const DialecticIsGizmoNumberKey;
extern NSString *const DialecticIsGoogleChatAccountKey;
extern NSString *const DialecticIsIOS4DeviceKey;
extern NSString *const DialecticIsIPhoneKey;
extern NSString *const DialecticITunesVolumeKey;
extern NSString *const DialecticKeyKey;
extern NSString *const DialecticLimitedGroupKey;
extern NSString *const DialecticLimitToSingleGroupKey;
extern NSString *const DialecticLiteralKey;
extern NSString *const DialecticLiteralDialMethodKey;
extern NSString *const DialecticLocalAreaCodesKey;
extern NSString *const DialecticLocalDialMethodKey;
extern NSString *const DialecticLocalInternationalPrefixKey;
extern NSString *const DialecticLocalPrefixKey;
extern NSString *const DialecticLocalServerKey;
extern NSString *const DialecticLocalSuffixKey;
extern NSString *const DialecticLocationKey;
extern NSString *const DialecticLocationNameKey;
extern NSString *const DialecticMainActionKey;
extern NSString *const DialecticMapServiceKey;
extern NSString *const DialecticMaximumInternalLengthKey;
extern NSString *const DialecticMaximumLocalLengthKey;
extern NSString *const DialecticModemDeviceKey;
extern NSString *const DialecticModificationDateKey;
extern NSString *const DialecticMuteKey;
extern NSString *const DialecticNameKey;
extern NSString *const DialecticNormalizeCompanyNamesKey;
extern NSString *const DialecticNoteKey;
extern NSString *const DialecticNoteActionKey;
extern NSString *const DialecticNumberKey;
extern NSString *const DialecticNumberOfRecentContactsKey;
extern NSString *const DialecticNumberOriginalKey;
extern NSString *const DialecticObjectClassKey;
extern NSString *const DialecticOutgoingKey;
extern NSString *const DialecticOutsidePrefixKey;
extern NSString *const DialecticOutsidePrefixEnabledKey;
extern NSString *const DialecticOverrideSystemVolumeKey;
extern NSString *const DialecticOverrideSystemVolumeLevelKey;
extern NSString *const DialecticPasswordKey;
extern NSString *const DialecticPauseDVDPlayerKey;
extern NSString *const DialecticPauseITunesKey;
extern NSString *const DialecticPhoneNumberKey;
extern NSString *const DialecticPreprocessNumberScriptKey;
extern NSString *const DialecticPriorityKey;
extern NSString *const DialecticPromptForDialPortWhenDialingKey;
extern NSString *const DialecticRecordingEnabledKey;
extern NSString *const DialecticRedialOnBusyKey;
extern NSString *const DialecticRedialOnBusyPauseKey;
extern NSString *const DialecticRedialOnBusyTotalAttemptsKey;
extern NSString *const DialecticRemoteKey;
extern NSString *const DialecticRemoteIPAddressKey;
extern NSString *const DialecticRemotePasswordKey;
extern NSString *const DialecticRemoteServerPortKey;
extern NSString *const DialecticRemoteUsernameKey;
extern NSString *const DialecticRequiresSSLKey;
extern NSString *const DialecticResetKey;
extern NSString *const DialecticScreenSaverActionKey;
extern NSString *const DialecticScriptPathKey;
extern NSString *const DialecticSeparateABMenuEnabledKey;
extern NSString *const DialecticServiceKey;
extern NSString *const DialecticServiceTypeKey;
extern NSString *const DialecticShowAlertKey;
extern NSString *const DialecticShowBroadcastStatusAlertKey;
extern NSString *const DialecticShowRestoreIAppDialogKey;
extern NSString *const DialecticSilenceDetectionEnabledKey;
extern NSString *const DialecticSilentKey;
extern NSString *const DialecticSIPDialMethodKey;
extern NSString *const DialecticSoftphoneKey;
extern NSString *const DialecticSortByLastNameKey;
extern NSString *const DialecticSoundEnabledKey;
extern NSString *const DialecticSpeakAlertKey;
extern NSString *const DialecticStatusStringKey;
extern NSString *const DialecticStripFormattingKey;
extern NSString *const DialecticSubmenuOrderKey;
extern NSString *const DialecticSuccessKey;
extern NSString *const DialecticSystemVolumeKey;
extern NSString *const DialecticTakeOffHookKey;
extern NSString *const DialecticTollFreeCodesKey;
extern NSString *const DialecticTollFreeDialMethodKey;
extern NSString *const DialecticTollFreePrefixKey;
extern NSString *const DialecticTollFreeSuffixKey;
extern NSString *const DialecticTouchtonesKey;
extern NSString *const DialecticUniqueIDKey;
extern NSString *const DialecticUrlActionKey;
extern NSString *const DialecticURLStringKey;
extern NSString *const DialecticUseModemKey;
extern NSString *const DialecticUseNewAuthenticationMethodKey;
extern NSString *const DialecticUsePregeneratedTonesKey;
extern NSString *const DialecticUseRemoteDialingRulesKey;
extern NSString *const DialecticUsernameKey;
extern NSString *const DialecticVerboseLoggingEnabledKey;
extern NSString *const DialecticWaitForDialToneKey;
extern NSString *const DialecticWaitForResponseKey;

//notification names
extern NSString *const DialecticIncomingCallDidStartNotification;
extern NSString *const DialecticIncomingCallDidEndNotification;
extern NSString *const DialecticOutgoingCallDidStartNotification;
extern NSString *const DialecticOutgoingCallDidEndNotification;

//error domains
extern NSString *const DialecticDialErrorDomain;
extern NSString *const DialecticIncomingCallAlertErrorDomain;
